package sun.java2d;

import java.util.Comparator;
import java.util.Iterator;
import java.util.Collections;
import java.util.Vector;
import java.util.List;

public class Spans
{
    private static final int kMaxAddsSinceSort = 256;
    private List mSpans;
    private int mAddsSinceSort;
    
    public Spans() {
        this.mSpans = new Vector(256);
        this.mAddsSinceSort = 0;
    }
    
    public void add(float n, float n2) {
        if (this.mSpans != null) {
            this.mSpans.add(new Span(n, n2));
            if (++this.mAddsSinceSort >= 256) {
                this.sortAndCollapse();
            }
        }
    }
    
    public void addInfinite() {
        this.mSpans = null;
    }
    
    public boolean intersects(float n, float n2) {
        int n3;
        if (this.mSpans != null) {
            if (this.mAddsSinceSort > 0) {
                this.sortAndCollapse();
            }
            n3 = (Collections.binarySearch(this.mSpans, new Span(n, n2), Spans.SpanIntersection.instance) >= 0);
        }
        else {
            n3 = true;
        }
        return n3;
    }
    
    private void sortAndCollapse() {
        Collections.sort(this.mSpans);
        this.mAddsSinceSort = 0;
        Iterator<Object> iterator = this.mSpans.iterator();
        Spans.Span span = null;
        if (iterator.hasNext()) {
            span = (Spans.Span)iterator.next();
        }
        while (iterator.hasNext()) {
            Spans.Span span2 = (Spans.Span)iterator.next();
            if (span.subsume(span2)) {
                iterator.remove();
            }
            else {
                span = span2;
            }
        }
    }
    
    static class Span implements Comparable
    {
        private float mStart;
        private float mEnd;
        
        Span(float mStart, float mEnd) {
            this.mStart = mStart;
            this.mEnd = mEnd;
        }
        
        final float getStart() {
            return this.mStart;
        }
        
        final float getEnd() {
            return this.mEnd;
        }
        
        final void setStart(float mStart) {
            this.mStart = mStart;
        }
        
        final void setEnd(float mEnd) {
            this.mEnd = mEnd;
        }
        
        boolean subsume(Spans.Span span) {
            int contains = this.contains(span.mStart);
            if (contains && span.mEnd > this.mEnd) {
                this.mEnd = span.mEnd;
            }
            return contains;
        }
        
        boolean contains(float n) {
            return this.mStart <= n && n < this.mEnd;
        }
        
        @Override
        public int compareTo(Object o) {
            float start = ((Spans.Span)o).getStart();
            int n;
            if (this.mStart < start) {
                n = -1;
            }
            else if (this.mStart > start) {
                n = 1;
            }
            else {
                n = 0;
            }
            return n;
        }
        
        @Override
        public String toString() {
            return "Span: " + this.mStart + " to " + this.mEnd;
        }
    }
    
    static class SpanIntersection implements Comparator
    {
        static final Spans.SpanIntersection instance;
        
        private SpanIntersection() {
        }
        
        @Override
        public int compare(Object o, Object o2) {
            Spans.Span span = (Spans.Span)o;
            Spans.Span span2 = (Spans.Span)o2;
            int n;
            if (span.getEnd() <= span2.getStart()) {
                n = -1;
            }
            else if (span.getStart() >= span2.getEnd()) {
                n = 1;
            }
            else {
                n = 0;
            }
            return n;
        }
        
        static {
            instance = new Spans.SpanIntersection();
        }
    }
}
